/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sound_esd.c,v 1.7 2003/03/03 02:16:34 tgucio Exp $
 */


#include <configure.h>	/* the file generated by configure */
#ifdef HAVE_ESD

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <pthread.h>
#include <esd.h>
/* local includes */
#include <defaults.h>
#include <sound.h>
#include <sound_esd.h>

int sound_fd;


/* initialize the esd sound support. */
int
sound_esd_init(void)
{
	/*
	** open the esd server.
	*/
	
	if (sound_esd_open() < 0)
	{
		DEBUG_MSG(DBG_WEIRD, "sound_esd_open() < 0, uuh?");
		return 0;
	}
		
	return 1;
}


/*
** shutdown the esd sound support.
** simply shut down the esd connection.
*/
void
sound_esd_shutdown(void)
{
	/* close the esd connection.  it should handle freeing loaded samples etc by itself. (i hope :)). */
	sound_esd_close();
	return;
}

/* setup a connection to the local ESD server. */
int
sound_esd_open(void)
{
	DEBUG_MSG(DBG_INFO, "*****************************");
	if ((sound_fd = esd_open_sound(NULL)) < 0) {
		SOUND_DEBUG("failed to open ESD sound device");
		return -1;
	}
	DEBUG_MSG(DBG_INFO, "-----------------------------");
	
	DEBUG_MSG(DBG_INFO, "opened sound_fd to %d", sound_fd);

	return 1;
}


/* simply closes the ESD connection.*/
int
sound_esd_close(void)
{
	esd_close(sound_fd);
	return 0;
}


/*
** Preload a file onto the esd and return the esd id given for the sound by the server.
*/
int
sound_esd_preload(char *name, char *path)
{
	int eid;

	if (!name || !path)
		return 0;

	/* cache the file (note for fun that those files should maybe not be too big). */
	eid = esd_file_cache(sound_fd, name, path);
	if (eid < 0)
		return 0;

	return eid;
}

/*
** Play event, for example when starting up.
*/
int
sound_esd_play_event(struct sound_list *snd)
{
	if (sound_fd <= 0) {
		SOUND_DEBUG("No connection open to an ESD server\n");
		return 0;
	}

	if (snd->load == S_PRELOAD) {
		DEBUG_MSG(DBG_INFO, "playing %s preloaded from ESD.", snd->path);
		DEBUG_MSG(DBG_INFO, "the sound fd is %d and the preload id is %d", sound_fd, snd->esd_id);
		esd_sample_play(sound_fd, snd->esd_id);
	} else {
		if (snd->path) {
			sound_esd_play_file(snd->path, NULL);
		}
	}
	
	return 1;
}

/*
** play a file.  Load it into the ESD cache, play it back and free it.
*/
int
sound_esd_play_file(char *path, cb callback)
{
	int eid;

	/* load the sound onto the cache. */
	if ( (eid = sound_esd_preload(path, path)) <= 0)
		return 0;

	/* play the sound and free it from the cache afterwards.  */
	esd_sample_play(sound_fd, eid);
	esd_sample_free(sound_fd, eid);
	if(callback)	callback();

	return 1;
}



#endif
