/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sound_vorbis.c,v 1.10 2003/03/03 02:16:34 tgucio Exp $
 */


#include <configure.h>  /* file generated by configure. */

#ifdef HAVE_VORBIS
#ifdef HAVE_ESD

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <pthread.h>
#include <esd.h>
#include <vorbis/vorbisfile.h>
/* local header files.  */
#include <defaults.h>
#include <sound.h>
#include <sound_vorbis.h>
#include <sound_esd.h>


int ogg_fd;
char pcmout[ESD_BUF_SIZE]; /* take 4k out of the data segment, not the stack */
pthread_t thread;
int vorbis_playing;
struct sound_vorbis_thread svt;

/*
** Preload an ogg-vorbis file.
** Used for the notification sounds part.
*/
int
sound_vorbis_preload(char *name, char *path)
{
	vorbis_info *vi = NULL;
	esd_format_t format = 0;
	int bits, channels, rate, mode, func;
	int sampleid;
	unsigned long size;

	memset(&svt, 0, sizeof(struct sound_vorbis_thread));
	if (!(svt.f = fopen(path, "r"))) {
		/* fail to open that file. */
		return 0;		
	}

	/* get the file size. */
	fseek(svt.f, 0, SEEK_END);
	size = ftell(svt.f);
	fseek(svt.f, 0, SEEK_SET);

	if (ov_open(svt.f, &svt.vf, NULL, 0) < 0)
		return 0;	/* invalid bitstream. (probably not ogg-vorbis file). */

	vi = ov_info(&svt.vf, -1);

	rate = vi->rate;
	bits = ESD_BITS16;
	mode = ESD_STREAM;
	func = ESD_PLAY;
	switch (vi->channels) {
		case 1:
			channels = ESD_MONO;
			break;
		default:
			channels = ESD_STEREO;
			break;
	}	

	format = bits | channels | mode | func;
	
	sampleid = esd_sample_cache(ogg_fd, format, rate, size, path);

	svt.operation = CACHE;
	svt.sock = ogg_fd;
	svt.close_after = 0;
	svt.arg = sampleid;
	
	/* Vorbisfile/ESD.  Read tha file and decode and cache on ESD. */
	pthread_create(&thread, NULL, (void *)sound_vorbis_update_thread, (void *)&svt);	

	/* return the id of the cached sound. */
	return sampleid;
}


/*
** Init the vorbis support.
** Simply opens up a connection to ESD.
*/
int
sound_vorbis_init()
{
	if ((ogg_fd = esd_open_sound(NULL)) < 0) {
		SOUND_DEBUG("sound_vorbis: failed to open ESD sound device\n");
		return 0;
	}
	
	return 1;
}

/*
** shutdown the vorbis support.
** simply closes the connection to esd.
*/
void
sound_vorbis_shutdown()
{
	esd_close(ogg_fd);
	ogg_fd = 0;
	return;
}



/*
** Play a sound event (for the notification sounds part).
*/
int
sound_vorbis_play_event(struct sound_list *snd)
{
	if (ogg_fd <= 0)
		return 0;

	esd_sample_play(ogg_fd, snd->esd_id);

	return 1;
}



/*
** Play a file encoded by ogg-vorbis.
*/
int
sound_vorbis_play_file(char *path, cb callback)
{
	vorbis_info *vi = NULL;
	esd_format_t format = 0;
	int bits, channels, rate, mode, func;
	unsigned long size;

	memset(&svt, 0, sizeof(struct sound_vorbis_thread));
	if (!(svt.f = fopen(path, "r"))) {
		/* fail to open that file. */
		return 0;		
	}

	/* get the file size. */
	fseek(svt.f, 0, SEEK_END);
	size = ftell(svt.f);
	fseek(svt.f, 0, SEEK_SET);

	if (ov_open(svt.f, &svt.vf, NULL, 0) < 0)
		return 0;	/* invalid bitstream. (probably not ogg-vorbis file). */

	/* extract some info out of the ogg. */
	vi = ov_info(&svt.vf, -1);

	/* setup the format for the ESD. */
	rate = vi->rate;
	DEBUG_MSG(DBG_INFO, "rate is %d", rate);
	bits = ESD_BITS16;
	mode = ESD_STREAM;
	func = ESD_PLAY;
	switch (vi->channels) {
		case 1:
			channels = ESD_MONO;
			break;
		default:
			channels = ESD_STEREO;
			break;
	}	

	DEBUG_MSG(DBG_INFO, "rate: %d bits: %d mode: %d func: %d channels: %d", rate, bits, mode, func, channels);
	
	format = bits | channels | mode | func;
/*	sampleid = esd_sample_cache(sound_fd, format, rate, size, path);*/
	svt.sock = esd_play_stream(format, rate, NULL, "junkie oggvorbis playback");
	if (svt.sock <= 0) {
		DEBUG_MSG(DBG_ERROR, "failed to connect to esd server");
		return 0;
	}

	svt.operation = PLAYBACK;
	svt.close_after = 1;
	
	svt.callback = callback;
	
	/* pull off a thread to handle the playback. */
	pthread_create(&thread, NULL, (void *)sound_vorbis_update_thread, (void *)&svt);
/*	sound_vorbis_update_thread(&svt);*/

	return 1;
}


int
sound_vorbis_stop()
{
	DEBUG_MSG(DBG_INFO, "sound_mpg stop got callled, mpg_playing zeroed");
	vorbis_playing = 0;
	return 1;
}
/*
** The thread that handles sending decoding and sending vorbis data to the
** ESD server.
*/
void
sound_vorbis_update_thread(void *arg)
{
	struct sound_vorbis_thread *vt = NULL;
	int length, current_section;

	if (!(vt = arg))
		return;

	vorbis_playing = 1;
	/* Vorbisfile/ESD.  Read tha file and decode and cache on ESD. */
	while (vorbis_playing) {
		if ((length = ov_read(&vt->vf, pcmout, ESD_BUF_SIZE, 0, 2, 1, &current_section)) <= 0) 
			break;
		
		/* fwrite(pcmout, 1, ret, stdout);   output this.. */
		length = write(vt->sock, pcmout, length);
		if (length <= 0)
			break;
	}	

	vorbis_playing = 0;
	/* clean up. */
	vt->ret = 1;
	if (vt->operation == CACHE) {
		if (esd_confirm_sample_cache(vt->sock) != vt->arg) 
			vt->ret = 0;	/* confirmation refused. */
	}

	/* clean up. */
	ov_clear(&vt->vf);
	/*fclose(vt->f); vorbisfile does the job for us. */
	if (vt->close_after)
		close(vt->sock);


	/* terminate the threed. */
	if(vt->callback)	vt->callback();
	pthread_exit(NULL);
	
	return;	
}


#endif
#endif
